@@ -1,6 +1,6 @@
 # extensions.py - extension handling for mercurial
 #
-# Copyright 2005-2007 Matt Mackall <mpm@selenic.com>
+# Copyright 2005-2007 Olivia Mackall <olivia@selenic.com>
 #
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
@@ -222,13 +222,16 @@ def load(ui, name, path, loadingtime=None):
     # extensions short circuit when loaded with a known incompatible version
     # of Mercurial.
     minver = getattr(mod, 'minimumhgversion', None)
-    if minver and util.versiontuple(minver, 2) > util.versiontuple(n=2):
-        msg = _(
-            b'(third party extension %s requires version %s or newer '
-            b'of Mercurial (current: %s); disabling)\n'
-        )
-        ui.warn(msg % (shortname, minver, util.version()))
-        return
+    if minver:
+        curver = util.versiontuple(n=2)
+
+        if None in curver or util.versiontuple(minver, 2) > curver:
+            msg = _(
+                b'(third party extension %s requires version %s or newer '
+                b'of Mercurial (current: %s); disabling)\n'
+            )
+            ui.warn(msg % (shortname, minver, util.version()))
+            return
     ui.log(b'extension', b'    - validating extension tables: %s\n', shortname)
     _validatetables(ui, mod)
 
@@ -454,7 +457,7 @@ def _loadextra(ui, newindex, extraloaders):
 
 
 def afterloaded(extension, callback):
-    '''Run the specified function after a named extension is loaded.
+    """Run the specified function after a named extension is loaded.
 
     If the named extension is already loaded, the callback will be called
     immediately.
@@ -464,7 +467,7 @@ def afterloaded(extension, callback):
 
     The callback receives the named argument ``loaded``, which is a boolean
     indicating whether the dependent extension actually loaded.
-    '''
+    """
 
     if extension in _extensions:
         # Report loaded as False if the extension is disabled
@@ -497,12 +500,12 @@ def populateui(ui):
 
 
 def bind(func, *args):
-    '''Partial function application
+    """Partial function application
 
-      Returns a new function that is the partial application of args and kwargs
-      to func.  For example,
+    Returns a new function that is the partial application of args and kwargs
+    to func.  For example,
 
-          f(1, 2, bar=3) === bind(f, 1)(2, bar=3)'''
+        f(1, 2, bar=3) === bind(f, 1)(2, bar=3)"""
     assert callable(func)
 
     def closure(*a, **kw):
@@ -615,7 +618,7 @@ class wrappedfunction(object):
 
 
 def wrapfunction(container, funcname, wrapper):
-    '''Wrap the function named funcname in container
+    """Wrap the function named funcname in container
 
     Replace the funcname member in the given container with the specified
     wrapper. The container is typically a module, class, or instance.
@@ -646,7 +649,7 @@ def wrapfunction(container, funcname, wrapper):
     work. Since you cannot control what other extensions are loaded by
     your end users, you should play nicely with others by using the
     subclass trick.
-    '''
+    """
     assert callable(wrapper)
 
     origfn = getattr(container, funcname)
@@ -665,7 +668,7 @@ def wrapfunction(container, funcname, wrapper):
 
 
 def unwrapfunction(container, funcname, wrapper=None):
-    '''undo wrapfunction
+    """undo wrapfunction
 
     If wrappers is None, undo the last wrap. Otherwise removes the wrapper
     from the chain of wrappers.
@@ -673,7 +676,7 @@ def unwrapfunction(container, funcname, wrapper=None):
     Return the removed wrapper.
     Raise IndexError if wrapper is None and nothing to unwrap; ValueError if
     wrapper is not None but is not found in the wrapper chain.
-    '''
+    """
     chain = getwrapperchain(container, funcname)
     origfn = chain.pop()
     if wrapper is None:
@@ -686,13 +689,13 @@ def unwrapfunction(container, funcname, wrapper=None):
 
 
 def getwrapperchain(container, funcname):
-    '''get a chain of wrappers of a function
+    """get a chain of wrappers of a function
 
     Return a list of functions: [newest wrapper, ..., oldest wrapper, origfunc]
 
     The wrapper functions are the ones passed to wrapfunction, whose first
     argument is origfunc.
-    '''
+    """
     result = []
     fn = getattr(container, funcname)
     while fn:
@@ -710,7 +713,7 @@ def _disabledpaths():
     # it might not be on a filesystem even if it does.
     if util.safehasattr(hgext, '__file__'):
         extpath = os.path.dirname(
-            os.path.abspath(pycompat.fsencode(hgext.__file__))
+            util.abspath(pycompat.fsencode(hgext.__file__))
         )
         try:
             files = os.listdir(extpath)
@@ -741,11 +744,11 @@ def _disabledpaths():
 
 
 def _moduledoc(file):
-    '''return the top-level python documentation for the given file
+    """return the top-level python documentation for the given file
 
     Loosely inspired by pydoc.source_synopsis(), but rewritten to
     handle triple quotes and to return the whole text instead of just
-    the synopsis'''
+    the synopsis"""
     result = []
 
     line = file.readline()
@@ -807,7 +810,7 @@ def disabled():
     exts = {}
     for name, path in pycompat.iteritems(paths):
         doc = _disabledhelp(path)
-        if doc:
+        if doc and name != b'__index__':
             exts[name] = doc.splitlines()[0]
 
     return exts
@@ -880,8 +883,8 @@ def _finddisabledcmd(ui, cmd, name, path, strict):
 
 
 def disabledcmd(ui, cmd, strict=False):
-    '''find cmd from disabled extensions without importing.
-    returns (cmdname, extname, doc)'''
+    """find cmd from disabled extensions without importing.
+    returns (cmdname, extname, doc)"""
 
     paths = _disabledpaths()
     if not paths:
@@ -909,6 +912,7 @@ def enabled(shortname=True):
     exts = {}
     for ename, ext in extensions():
         doc = gettext(ext.__doc__) or _(b'(no help text available)')
+        assert doc is not None  # help pytype
         if shortname:
             ename = ename.split(b'.')[-1]
         exts[ename] = doc.splitlines()[0].strip()
@@ -926,13 +930,21 @@ def notloaded():
 def moduleversion(module):
     '''return version information from given module as a string'''
     if util.safehasattr(module, b'getversion') and callable(module.getversion):
-        version = module.getversion()
+        try:
+            version = module.getversion()
+        except Exception:
+            version = b'unknown'
+
     elif util.safehasattr(module, b'__version__'):
         version = module.__version__
     else:
         version = b''
     if isinstance(version, (list, tuple)):
         version = b'.'.join(pycompat.bytestr(o) for o in version)
+    else:
+        # version data should be bytes, but not all extensions are ported
+        # to py3.
+        version = stringutil.forcebytestr(version)
     return version
 
 
