@@ -40,6 +40,8 @@ if not ispy3:
     def future_set_exception_info(f, exc_info):
         f.set_exception_info(*exc_info)
 
+    # this is close enough for our usage
+    FileNotFoundError = OSError
 
 else:
     import concurrent.futures as futures
@@ -53,6 +55,8 @@ else:
     def future_set_exception_info(f, exc_info):
         f.set_exception(exc_info[0])
 
+    FileNotFoundError = __builtins__['FileNotFoundError']
+
 
 def identity(a):
     return a
@@ -335,7 +339,7 @@ if ispy3:
     def strkwargs(dic):
         """
         Converts the keys of a python dictonary to str i.e. unicodes so that
-        they can be passed as keyword arguments as dictonaries with bytes keys
+        they can be passed as keyword arguments as dictionaries with bytes keys
         can't be passed as keyword arguments to functions on Python 3.
         """
         dic = {k.decode('latin-1'): v for k, v in dic.items()}
@@ -343,7 +347,7 @@ if ispy3:
 
     def byteskwargs(dic):
         """
-        Converts keys of python dictonaries to bytes as they were converted to
+        Converts keys of python dictionaries to bytes as they were converted to
         str to pass that dictonary as a keyword argument on Python 3.
         """
         dic = {k.encode('latin-1'): v for k, v in dic.items()}
@@ -506,7 +510,19 @@ def mkstemp(suffix=b'', prefix=b'tmp', dir=None):
     return tempfile.mkstemp(suffix, prefix, dir)
 
 
-# mode must include 'b'ytes as encoding= is not supported
+# TemporaryFile does not support an "encoding=" argument on python2.
+# This wrapper file are always open in byte mode.
+def unnamedtempfile(mode=None, *args, **kwargs):
+    if mode is None:
+        mode = 'w+b'
+    else:
+        mode = sysstr(mode)
+    assert 'b' in mode
+    return tempfile.TemporaryFile(mode, *args, **kwargs)
+
+
+# NamedTemporaryFile does not support an "encoding=" argument on python2.
+# This wrapper file are always open in byte mode.
 def namedtempfile(
     mode=b'w+b', bufsize=-1, suffix=b'', prefix=b'tmp', dir=None, delete=True
 ):
